#!/usr/bin/env python3
import asyncio
import time

import aiohttp


async def look_up():
    async with aiohttp.request('GET', 'https://dictionary.cambridge.org/dictionary/english/get') as resp:
        text = await resp.text()
        print(text)


def main():
    loop = asyncio.get_event_loop()
    task = look_up()
    loop.run_until_complete(task)


if __name__ == '__main__':
    now = time.time()
    main()
    end = time.time()
    print('using time ' + str(end - now))



# run this test script in cambrinary dir.
import argparse
import json
import logging
import subprocess
import sys
import time
from argparse import RawTextHelpFormatter
from collections import OrderedDict

logging.basicConfig()
logger = logging.getLogger('logger')
logger.setLevel(logging.INFO)

sys.path.append('.')  # for python3 test/test.py
from cambrinary.country_const import *


def get_args():
    parser = argparse.ArgumentParser(description='test for cambrinary', formatter_class=RawTextHelpFormatter)
    parser.add_argument('-t', '--translation', action='store',
                        help="Prefered language to test. eg: python3 test/test.py -t")
    args = parser.parse_args()
    return args


def load(json_file):
    with open(json_file) as f:
        return json.load(f, object_pairs_hook=OrderedDict)


test_case = load('test/test_case.json')


def run_case(word, trans=None):
    time.sleep(2)  # avoid http request block
    trans_arg = ''
    if trans:
        trans_arg = '-t {} '.format(trans)
    cmd = 'cambrinary {}-w {}'.format(trans_arg, word)
    try:
        if trans:
            logger.info("TEST word " + word + " in " + trans)
        else:
            logger.info("TEST word " + word + " in default")
        subprocess.check_output(cmd, shell=True)
    except Exception:
        logger.error("error occur when exec command: " + cmd)


def run_test(lang):
    logger.info('Comming to test language: ' + lang)
    alphabet = test_case['alphabet']
    for a in alphabet:
        run_case(a, lang)
    words = test_case['words']
    for w in words:
        run_case(w, lang)
    phrases = test_case['phrases']
    for p in phrases:
        run_case(p, lang)


def main():
    args = get_args()
    if args.translation:
        if args.translation not in SUPPORT_LANG:
            logger.info('Not support this language yet.')
        run_test(args.translation)
        return
    for lang in SUPPORT_LANG:
        run_test(lang)


if __name__ == '__main__':
    main()



GB = 'english'
CN = 'chinese'
DE = 'german'
JP = 'japanese'
FR = 'french'
RU = 'russian'
IT = 'italian'

SUPPORT_LANG = [GB, CN, DE, JP, FR, IT]  # no support for russian temporarily due to irregular HTML tags.



#!/usr/bin/env python3
import argparse
import asyncio
from argparse import RawTextHelpFormatter

import aiohttp
from bs4 import BeautifulSoup

from .country_const import *
from .log import *
from .type import *

translation = {GB: 'english',
               CN: 'english-chinese-traditional',
               DE: 'english-german',
               JP: 'english-japanese',
               FR: 'english-french',
               RU: 'english-russian',
               IT: 'english-italian'}


def get_args():
    parser = argparse.ArgumentParser(description='A linux online terminal dictionary based on cambridge dictionary',
                                     formatter_class=RawTextHelpFormatter)
    parser.add_argument('-w', '--word', required=True, nargs='+', action='store', help='the word you want to look up')
    parser.add_argument('-t', '--translation', action='store',
                        help="Prefered language to explain. Defult(english explaination)\n"
                             "Supported language:\n"
                             "{}".format(''.join(['- ' + lang + '\n' for lang in SUPPORT_LANG])))
    parser.add_argument('-s', '--synonym', action="store_true", help='show the synonym of the word if has')
    args = parser.parse_args()
    return args


async def load(word, trans):
    url = 'https://dictionary.cambridge.org/dictionary/' + trans + '/' + word
    async with aiohttp.request('GET', url) as resp:
        html = await resp.text()
        return html
    # from urllib import request
    # response = request.urlopen(url)
    # html = response.read()
    # return html


def parse_xref(xref, indent=4):
    """
    parse things like synonym, idioms
    :param xref: synonym or idioms object
    :param indent:
    :return: parsed string
    """
    res = ''
    items = xref.findAll('div', attrs={'class': 'item'})
    for i in items:
        res += '{}{}\n'.format('' * indent, i.get_text())
    return res


def parse_pronunciation(part_speech, trans):
    """
    retrieve the pronunciation from part_speech
    :param part_speech:
    :param trans:
    :return:
    """
    pronunciation = Pronunciation()
    if trans == GB or trans == CN:
        header = part_speech.find('div', attrs={'class': 'pos-header'})

        pos = header.findAll('span', attrs={'class': 'pos'})
        gcs = header.find('span', attrs={'class': 'gcs'})
        if pos:
            pronunciation.pos = '{}'.format(', '.join([p.get_text() for p in pos]))
        if gcs:
            pronunciation.gcs = gcs.get_text().strip()
        prons = header.findAll('span', recursive=False)
        res = ''
        for p in prons:
            region = p.find('span', attrs={'class': 'region'})
            # lab = p.find('span', attrs={'class': 'lab'})
            ipa = p.find('span', attrs={'class': 'ipa'})
            if region and ipa:
                res += '{} {} '.format(
                    colors.pron_region(region.get_text().upper()),  # lab.get_text().upper() + ' ' if lab else '',
                    colors.pronunciation('/{}/'.format(ipa.get_text())))
        pronunciation.prons = res
    if trans == DE or trans == FR:
        di_info = part_speech.find('span', attrs={'class': 'di-info'})
        if di_info:  # like look-up has no pronunciation
            pos = di_info.find('span', attrs={'class': 'pos'})
            ipa = di_info.find('span', attrs={'class': 'ipa'})
            if pos:
                pronunciation.pos = pos.get_text()
            if ipa:
                pronunciation.prons = colors.pronunciation(' /{}/ '.format(ipa.get_text()))
    if trans == JP or trans == IT:
        header = part_speech.find('div', attrs={'class': 'pos-header'})
        if not header:  # word look-up in japanese
            header = part_speech.find('span', attrs={'class': 'di-info'})
        pos = header.find('span', attrs={'class': 'pos'})
        gcs = header.find('span', attrs={'class': 'gcs'})
        pron_info_list = header.findAll('span', attrs={'class': 'pron-info'})
        if pos:
            pronunciation.pos = pos.get_text()
        if gcs:
            pronunciation.gcs = gcs.get_text().strip()
        if pron_info_list:
            res = ''
            for pron in pron_info_list:
                region = pron.find('span', attrs={'class': 'region'})
                ipa = pron.find('span', attrs={'class': 'ipa'})
                if region and ipa:
                    res += '{} {} '.format(colors.pron_region(region.get_text().upper()),
                                           colors.pronunciation('/{}/'.format(ipa.get_text())))
            pronunciation.prons = res
    if trans == RU:
        header = part_speech.find('span', attrs={'class': 'di-info'})
        pos = header.find('span', attrs={'class': 'pos'})
        gcs = header.find('span', attrs={'class': 'gcs'})
        pron_info_list = header.findAll('span', attrs={'class': 'pron-info'})
        if pos:
            pronunciation.pos = pos.get_text()
        if gcs:
            pronunciation.gcs = gcs.get_text().strip()
        if pron_info_list:
            res = ''
            for pron in pron_info_list:
                region = pron.find('span', attrs={'class': 'region'})
                ipa = pron.find('span', attrs={'class': 'ipa'})
                if region and ipa:
                    res += '{} {} '.format(colors.pron_region(region.get_text().upper()),
                                           colors.pronunciation('/{}/'.format(ipa.get_text())))
            pronunciation.prons = res
    logger.info('the pronunciation is: {}'.format(pronunciation.to_str()))
    return pronunciation


def get_sense_block_title(block):
    """
    :param block: html element BeautifulSoup object
    :return: string
    """
    res = None
    dsense_h = block.find('h3', attrs={'class': 'dsense_h'})
    if dsense_h:
        logger.info('fetch the dsense_h')
        pos = dsense_h.find('span', attrs={'class': 'pos dsense_pos'})
        guideword = dsense_h.find('span', attrs={'class': 'guideword dsense_gw'})
        sub_word = guideword.find('span')
        res = '{} {}\n'.format(colors.guidword('[' + sub_word.get_text() + ']'), pos.get_text() if pos else '')
    # case for russian, words like 'get'
    sense_head = block.find('span', attrs={'class': 'sense-head'})
    if sense_head:
        guideword = sense_head.find('strong', attrs={'class': 'gw'})
        gc = sense_head.find('span', attrs={'class': 'gc'})
        res = '{} {}\n'.format(colors.guidword(guideword.get_text()), '[' + gc.get_text() + ']' if gc else '')
    logger.info('the  title of block is {}'.format(res))
    return res


def parse_pad_indents(block, args):
    res = []
    sense_body = block.find('div', attrs={'class': 'sense-body dsense_b'})
    pad_indents = sense_body.findAll('div', attrs={'class': 'def-block ddef_block'})
    logger.info('the number of pad_indent is {}'.format(len(pad_indents) if pad_indents else 0))

    def get_definition(p):
        d = p.find('div', attrs={'class': 'def ddef_d db'})
        return d.get_text() if d else None

    def get_trans(body):
        trans = body.find('span', attrs={'class': 'trans dtrans dtrans-se'}, recursive=False)
        return trans.get_text().strip() if trans else None

    def get_examps(body):
        examps = body.findAll('span', attrs={'class': 'eg deg'})
        return [e.get_text().strip() for e in examps] if examps else None

    def get_synonym(body, arg):
        if arg.synonym:
            synonym = body.find('div', attrs={'class': 'xref synonym'})
            if synonym:
                pad_indent.synonym = parse_xref(synonym)

    for pad in pad_indents:
        def_body = pad.find('span', attrs={'class': 'def-body ddef_b'})
        if not def_body:  # german dictionary use div tag
            def_body = pad.find('div', attrs={'class': 'def-body'})

        pad_indent = PadIndent(
            definition=get_definition(pad),
            trans=(get_trans(def_body) if def_body else None),
            examps=(get_examps(def_body) if def_body else None),
            synonym=(get_synonym(def_body, args)) if def_body else None
        )
        res.append(pad_indent)

    return res


def parse_block(block):
    """
    a block has multi def-blocks which give a detail definition fo the word
    :param block:
    :return:
    """
    args = get_args()

    sense_b = SenseBlock(
        title=get_sense_block_title(block),
        pad_indents=parse_pad_indents(block, args)
    )
    return sense_b


def get_part_speeches(dict, trans):
    """
    'help' has verb and noun at the same time. So each of them for one part of speech(pos)
    :param dict: dictionary
    :param trans: translation language
    :return: a list of pos
    """
    if trans == GB or trans == CN:
        return dict.findAll('div', attrs={'class': 'pr entry-body__el'}) or \
               dict.findAll('div', attrs={'class': 'entry-body__el clrd js-share-holder'})
    if trans == DE:
        return dict.findAll('div', attrs={'class': 'd pr di english-german kdic'})
    if trans == JP:
        return dict.findAll('div', attrs={'class': 'pr entry-body__el'})
    if trans == FR:
        return dict.findAll('div', attrs={'class': 'd pr di english-french kdic'})
    if trans == RU or trans == IT:
        return dict.findAll('div', attrs={'class': 'pr entry-body__el'}) or \
               dict.findAll('div', attrs={
                   'class': 'di $dict entry-body__el entry-body__el--smalltop clrd js-share-holder'})  # for look-up case


def parse_sense_blocks(part_speech):
    sense_blocks = []
    blocks = []
    blocks_with_gw = part_speech.findAll('div', attrs={'class': 'pr dsense'})  # meaning groups with guide word
    blocks.extend(blocks_with_gw)
    blocks_no_gw = part_speech.findAll('div', attrs={'class': 'pr dsense dsense-noh'})  # no guide word
    blocks.extend(blocks_no_gw)
    # some language like german or french has no 'pr dsense' or 'pr dsense dsense-noh', so use 'sense-block' to catch
    # the block
    if not blocks:
        blocks.extend(part_speech.findAll('div', attrs={'class': 'sense-block pr dsense dsense-noh'}))
    logger.info('the number of block is {}'.format(len(blocks) if blocks else 0))
    for j, block in enumerate(blocks):
        logger.info('parse the {}th block'.format(j))
        sense_block = parse_block(block)
        sense_blocks.append(sense_block)
    return sense_blocks


def parse_part_speeches(part_speeches, trans):
    res = []
    for i, p in enumerate(part_speeches):
        logger.info('parse the {}th part_speech'.format(i))
        part_speech = PartSpeech(
            h_pronunciation=parse_pronunciation(p, trans),
            sense_blocks=parse_sense_blocks(p)
        )
        res.append(part_speech)
    return res


def get_dictionary(html):
    """
    retrieve dictionary body.
    :param html:
    :return:
    """
    pr_dictionary = 'pr dictionary'
    entry_body = 'entry-body'
    parsed_html = BeautifulSoup(html, features='html.parser')
    # this area contains all the dictionaries
    res_dict = None
    dictionaries = parsed_html.body.findAll('div', attrs={'class': pr_dictionary})
    if dictionaries:  # get at least one dictionary

        res_dict = dictionaries[0]
        logger.info('get a dictionary by {}'.format(pr_dictionary))
    else:  # no dictionary, just entry-body
        res_dict = parsed_html.body.find('div', attrs={'class': entry_body})
        logger.info('get a dictionary by {}'.format(entry_body))
    return res_dict


async def look_up(word, trans, results):
    logger.info('begin to retrieve word: [{}] in {}'.format(word, trans))
    html = await load(word, translation[trans])
    dictionary = get_dictionary(html)
    if not dictionary:
        results[word] = 'No result for {}'.format(word)
        logger.info('No result for {}'.format(word))
        return

    part_speeches = get_part_speeches(dictionary, trans)
    logger.info('the number of part_speeh is {}'.format(len(part_speeches) if part_speeches else 0))
    word_obj = Word(
        part_speeches=parse_part_speeches(part_speeches, trans)
    )
    results[word] = word_obj.to_str()


def main():
    args = get_args()
    trans = conf.default_trans_language
    if args.translation:
        if args.translation not in translation:
            print('Not support in {} translation. Supported languages are:\n{}'.format(args.translation, ''.join(
                ['- ' + lang + '\n' for lang in SUPPORT_LANG])))
            logger.info('Not support in {} translation.'.format(args.translation))
            exit()
        trans = args.translation
    return_dict = OrderedDict()
    loop = asyncio.get_event_loop()
    tasks = []
    logger.info('start to work...')
    for w in args.word:
        return_dict[w] = None  # guarantee the orders
        tasks.append(look_up(w, trans, return_dict))
    loop.run_until_complete(asyncio.wait(tasks))

    # threading.Thread()
    # return_dict = OrderedDict()
    # for w in args.word:
    #     t = threading.Thread(target=look_up, name=w, args=(w, args.translation, args.dictionary, return_dict,))
    #     t.start()
    #     t.join()

    # using multiprocessing is almost the same time consume under around 7 words test, about 16 second.
    # also, the manager.dict() is a disorder dict not as good as the OrderedDict()
    # import multiprocessing
    # manager = multiprocessing.Manager()
    # return_dict = manager.dict()
    # for w in args.word:
    #     p = multiprocessing.Process(target=look_up, name=w, args=(w, args.translation, args.dictionary, return_dict,))
    #     p.start()
    #     p.join()

    for word, res in return_dict.items():
        if len(return_dict) > 1:
            print('------{}------'.format(word))
        print(res)


if __name__ == '__main__':
    # now = time.time()
    main()
    # end = time.time()
    # print('using time ' + str(end - now))



from .Conf import *


class Pron(object):
    def __init__(self):
        self.region = None
        self.lab = None
        self.ipa = None

    def to_str(self):
        pass


class Pronunciation(object):
    def __init__(self):
        self.pos = None
        self.gcs = None
        self.prons = None

    def to_str(self):
        res = ''
        if self.pos:
            res += '{} '.format(self.pos)
        if self.gcs:
            res += '[{}] '.format(self.gcs)
        if self.prons:
            res += self.prons
        return res if res == '' else res + '\n'


class PadIndent(object):
    def __init__(self, definition=None, trans=None, examps=None, synonym=None):
        self.definition = definition
        self.trans = trans
        self.examps = examps
        self.synonym = synonym

    def to_str(self):
        res = ''
        if self.definition:
            res += colors.definition('* ' + self.definition) + '\n'
        if self.trans:
            res += colors.trans_def('  {}\n'.format(self.trans))
        if self.examps:
            res += ''.join([colors.exam_sen('  - ' + e) + '\n' for e in self.examps])
        if self.synonym:
            res += '  Synonym' + '\n'
            res += self.synonym
        return res


class SenseBlock(object):
    def __init__(self, title, pad_indents):
        self.title = title  # could be None
        self.pad_indents = pad_indents  # []

    def to_str(self):
        res = ''
        if self.title:
            res += self.title
        for p in self.pad_indents:
            res += p.to_str()
        return res


class PartSpeech(object):
    def __init__(self, h_pronunciation, sense_blocks):
        self.h_pronunciation = h_pronunciation  # header
        self.sense_blocks = sense_blocks  # body []

    def to_str(self):
        res = ''
        if self.h_pronunciation:
            res += self.h_pronunciation.to_str()
        for s in self.sense_blocks:
            res += s.to_str()
        return res


class Word(object):
    def __init__(self, part_speeches):
        self.part_speeches = part_speeches  # [] for example, help is a verb or noun, each of them is a part_speech

    def to_str(self):
        res = ''
        for p in self.part_speeches:
            res += p.to_str()
        return res



# for reference: https://misc.flogisoft.com/bash/tip_colors_and_formatting
format_dic = {"default": 0,
              "bold": 1,
              "dim": 2,
              "underlined": 4,
              "blink": 5,
              "reverse": 7,
              "hidden": 8,
              }
foreground_dic = {"default": 39,
                  "black": 30,
                  "red": 31,
                  "green": 32,
                  "yellow": 33,
                  "blue": 34,
                  "magenta": 35,
                  "cyan": 36,
                  "light_gray": 37,
                  "dark_gray": 90,
                  "light_red": 91,
                  "light_green": 92,
                  "light_yellow": 93,
                  "light_blue": 94,
                  "light_magenta": 95,
                  "light_cyan": 96,
                  "white": 97
                  }
background_dic = {"default": 49,
                  "black": 40,
                  "red": 41,
                  "green": 42,
                  "yellow": 43,
                  "blue": 44,
                  "magenta": 45,
                  "cyan": 46,
                  "light_gray": 47,
                  "dark_gray": 100,
                  "light_red": 101,
                  "light_green": 102,
                  "light_yellow": 103,
                  "light_blue": 104,
                  "light_magenta": 105,
                  "light_cyan": 106,
                  "white": 107
                  }



import json
import os
from collections import OrderedDict
from pathlib import Path

from .color_const import *


def load(json_file):
    with open(json_file) as f:
        return json.load(f, object_pairs_hook=OrderedDict)


class ColorScheme(object):
    def __init__(self, pron_region, definition, exam_sen, trans_def, pronunciation, guidword):
        self.pron_region = pron_region
        self.definition = definition
        self.exam_sen = exam_sen
        self.trans_def = trans_def
        self.pronunciation = pronunciation
        self.guidword = guidword

    def to_dic(self):
        json_dic = OrderedDict()
        json_dic['pron_region'] = self.pron_region
        json_dic['definition'] = self.definition
        json_dic['exam_sen'] = self.exam_sen
        json_dic['trans_def'] = self.trans_def
        json_dic['pronunciation'] = self.pronunciation
        json_dic['guidword'] = self.guidword
        return json_dic

    @staticmethod
    def to_obj(color_scheme_dict):
        return ColorScheme(color_scheme_dict['pron_region'], color_scheme_dict['definition'],
                           color_scheme_dict['example_sentence'], color_scheme_dict['trans_definition'],
                           color_scheme_dict['pronunciation'], color_scheme_dict['guidword'])


class Conf(object):
    def __init__(self, path):
        self.conf_dict = load(path)
        self.color_scheme = ColorScheme.to_obj(self.conf_dict['color_scheme'])
        self.default_trans_language = self.conf_dict['default_trans_language']

    def to_dic(self):
        json_dic = OrderedDict()
        json_dic['color_scheme'] = self.color_scheme
        json_dic['default_trans_language'] = self.default_trans_language
        return json_dic


class Color(object):
    def __init__(self, color_scheme):
        self.color_scheme = color_scheme

    @staticmethod
    def color(s, color_scheme):
        """
        for reference: https://misc.flogisoft.com/bash/tip_colors_and_formatting
        :param s: str to color
        :param color_scheme: refer it in `color_const.py`
        :return: colored str
        """
        pre = '\033['
        post = '\033[0m'
        return '{}{};{};{}m{}{}'.format(pre, format_dic[color_scheme[0]], foreground_dic[color_scheme[1]],
                                        background_dic[color_scheme[2]], s, post)

    def pron_region(self, str):
        return self.color(str, self.color_scheme.pron_region)

    def pronunciation(self, str):
        return self.color(str, self.color_scheme.pronunciation)

    def definition(self, str):
        return self.color(str, self.color_scheme.definition)

    def trans_def(self, str):
        return self.color(str, self.color_scheme.trans_def)

    def exam_sen(self, str):
        return self.color(str, self.color_scheme.exam_sen)

    def guidword(self, str):
        return self.color(str, self.color_scheme.guidword)


try:
    cfg = Path(os.environ['XDG_CONFIG_HOME']).absolute() / 'cambrinary' / 'conf.json'
except KeyError:
    cfg = Path(os.environ['HOME']).absolute() / '.config' / 'cambrinary' / 'conf.json'
if not cfg.is_file():
    cfg = Path(__file__).absolute().parent / 'conf.json'
conf = Conf(str(cfg))
colors = Color(conf.color_scheme)



#!/usr/bin/env python
# setup.py generated by flit for tools that don't yet use PEP 517

from distutils.core import setup

packages = \
['cambrinary']

package_data = \
{'': ['*']}

install_requires = \
['aiohttp', 'beautifulsoup4']

extras_require = \
{'publish': ['flit']}

entry_points = \
{'console_scripts': ['cambrinary = cambrinary.cambrinary:main']}

setup(name='cambrinary',
      version='0.1.1',
      description='A Linux terminal online dictionary, based on `Cambridge Dictionary <https://dictionary.cambridge.org>`_',
      author='xueyuanl',
      author_email='p.xueyuanl@gmail.com',
      url='https://github.com/xueyuanl/cambrinary',
      packages=packages,
      package_data=package_data,
      install_requires=install_requires,
      extras_require=extras_require,
      entry_points=entry_points,
      python_requires='>=3.5.3',
     )



import os
import logging
from pathlib import Path


try:
    data = Path(os.environ['XDG_DATA_HOME']).absolute() / 'cambrinary'
except KeyError:
    data = Path(os.environ['HOME']).absolute() / '.local' / 'share' / 'cambrinary'
data.mkdir(parents=True, exist_ok=True)

logging.basicConfig(filename=str(data / 'dict.log'),
                    filemode='a',
                    format='%(asctime)s - %(pathname)s[line:%(lineno)d] - %(levelname)s: %(message)s',
                    level=logging.DEBUG)

logger = logging.getLogger('cambrinary')
